﻿package vfl.as3.scroll
{
	import flash.display.MovieClip;
	import flash.display.DisplayObjectContainer;
	import flash.geom.Point;
	import flash.events.Event;
	
	import flash.display.Sprite;
	import flash.filters.BitmapFilterQuality;
	import flash.filters.BlurFilter;

	
	public class AutoScroll extends MovieClip
	{	
		private var move_speed	: Number;
		
		private var target		: MovieClip = null;
		private var movieMask	: MovieClip = null;
		private var movieParent	: DisplayObjectContainer = null;
		
		public var horizontalEnabled	: Boolean = true;
		public var verticalEnabled		: Boolean = true;
		
		private var maxPos 	= null;
		private var maxBlur;
		private var hitArea_mc = null;
		
		public const damping = 0.2;
		
		
		public function AutoScroll(pTarget	: MovieClip,
								   pMask	: MovieClip,
								   pSpeed	: Number = 3,
								   pMaxBlur = 30) 
		{
			
			target 		= pTarget;
			movieMask	= pMask;
			move_speed	= pSpeed;
			
			movieParent	= target.parent;
			movieMask.x = movieMask.y = target.x = target.y = 0;
			
			addChild(target);
			addChild(movieMask);
			target.mask = movieMask;		
			
			
			maxBlur= pMaxBlur;
			maxPos = new Point( movieMask.width  - target.width,
							    movieMask.height - target.height );
			
			CreateListeners();
		}
		
		private function CreateListeners()
		{
			target.addEventListener(Event.ENTER_FRAME, UpdatePosition);
		}
		
		public function Update()
		{
			maxPos = new Point( movieMask.width  - target.width,
							    movieMask.height - target.height );			
		}
		
		private function UpdatePosition(evt)
		{	
			if (hitArea_mc == null) hitArea_mc = movieMask;
			var rect 		= hitArea_mc.getBounds(movieParent);
			
			var mousePos	= new Point(movieParent.mouseX - rect.x,
										movieParent.mouseY - rect.y);
			var speed		= new Point(0, 0);
			var diff;
			
			if (mousePos.x < 0 ||
				mousePos.x > hitArea_mc.width ||
				mousePos.y < 0 ||
				mousePos.y > hitArea_mc.height)
			{
				diff = new Point(0,0);
			}
			else
			{
				var mid		= new Point(rect.width/2, rect.height/2);
				var p 		= new Point( mousePos.x / rect.width,
										 mousePos.y / rect.height);
				/*
				var newPos = new Point();
				
				if (mousePos.x < 0) newPos.x = 0;
				else if (mousePos.x > hitArea_mc.width) newPos.x = maxPos.x;
				else  newPos.x = p.x * maxPos.x;
				
				if (mousePos.y < 0) newPos.y = 0;
				else if (mousePos.y > hitArea_mc.height) newPos.y = maxPos.y;
				else  newPos.y = p.y * maxPos.y;				
				*/
				
				
				var newPos  = new Point(p.x * maxPos.x,
										p.y * maxPos.y);
										
				
				diff	= new Point(	newPos.x - target.x,
										newPos.y - target.y  );
				
			}
	
			if (!horizontalEnabled) diff.x = 0;
			if (!verticalEnabled) diff.y = 0;
			
			var blur:BlurFilter = new BlurFilter();
			blur.quality 		= BitmapFilterQuality.MEDIUM;	
	
			if (Math.abs(diff.x) > 2)
			{
				blur.blurX 			= Math.abs((diff.x  / maxPos.x) * maxBlur);
				target.x = target.x + (diff.x * damping);
			}
			else blur.blurX 		= 0;
							
			if (maxPos.y > 0)
			{
				target.y = maxPos.y/2;
				blur.blurY			= 0;
			}
			else
			{
				if (Math.abs(diff.y) > 2)
				{
					blur.blurY 			= Math.abs((diff.y  / maxPos.y) * maxBlur);
					target.y = target.y + (diff.y * damping);
				}					
				else blur.blurY			= 0;
			}
			
			target.filters = [blur];
		}
		
		public function SetSpeed(pSpeed)
		{
			move_speed = pSpeed;
		}
		
		public function SetHitArea(pHitArea_mc)
		{
			hitArea_mc = pHitArea_mc;
			hitArea_mc.visible = false;
		}
		
		public function EnableHorizontalScroll()
		{
			horizontalEnabled = true;
		}
		
		public function DisableHorizontalScroll()
		{
			horizontalEnabled = false;
		}
		
		public function EnableVerticalScroll()
		{
			verticalEnabled = true;
		}
		
		public function DisableVerticalScroll()
		{
			verticalEnabled = false;
		}
		
		public function Dispose()
		{
			target.removeEventListener(Event.ENTER_FRAME, UpdatePosition);
		}
	}
}